@@ -1,5 +1,8 @@
+import re
 from copy import copy
+from dataclasses import dataclass, field, fields
 from datetime import datetime, timedelta, timezone
+from enum import Enum
 from itertools import groupby
 from typing import Optional
 from urllib.parse import parse_qs, urlsplit, quote as urlquote
@@ -7,9 +10,10 @@ from urllib.parse import parse_qs, urlsplit, quote as urlquote
 # import requests  # we use lazy imports to improve load time for local commands
 # from bs4 import BeautifulSoup
 # from bs4.element import NavigableString
+import click
 from tqdm import tqdm
 
-from kks.errors import APIError
+from kks.errors import APIError, ParseError
 from kks.util.h2t import HTML2Text
 
 
@@ -28,7 +32,7 @@ GROUP_ID_BY_CONTEST = {
 
 
 PROBLEM_INFO_VERSION = 3
-SERVER_TZ = timezone(timedelta(hours=0))  # UTC
+TIME_FORMAT = '%Y/%m/%d %H:%M:%S'
 MSK_TZ = timezone(timedelta(hours=3))
 
 
@@ -36,13 +40,18 @@ class Links:
     HOST = 'caos.myltsev.ru'
     CGI_BIN = f'https://{HOST}/cgi-bin'
     WEB_CLIENT_ROOT = f'{CGI_BIN}/new-client'
-    _template = f'{WEB_CLIENT_ROOT}?SID=__SID__&action='
-    SETTINGS = f'{_template}143'
-    SUMMARY = f'{_template}137'
-    SUBMISSIONS = f'{_template}140'
-    USER_STANDINGS = f'{_template}94'
-    SUBMIT_CLAR = f'{_template}141'
-    CLARS = f'{_template}142'
+
+
+class Page(Enum):
+    MAIN_PAGE = 2
+    VIEW_SOURCE = 36
+    DOWNLOAD_SOURCE = 91
+    USER_STANDINGS = 94
+    SUMMARY = 137
+    SUBMISSIONS = 140
+    SUBMIT_CLAR = 141
+    CLARS = 142
+    SETTINGS = 143
 
 
 class Status:
@@ -122,6 +131,11 @@ class ProblemWithDeadline:
 
 
 class CacheKeys:
+    problem_links = 'problem_links'
+    full_scores = 'full'
+    run_penalties = 'run'
+    server_tz = 'server_tz'
+
     @staticmethod
     def penalty(contest):
         return f'p_{contest}'
@@ -131,16 +145,91 @@ class CacheKeys:
         return f'dl_{contest}'
 
 
+def _skip_field(parser=None):
+    meta = {'skip': True}
+    if parser is not None:
+        meta['parser'] = parser
+    return field(init=False, repr=False, compare=False, metadata=meta)
+
+
+def _parse_field(parser):
+    return field(metadata={'parser': parser})
+
+
+class _CellParsers:
+    @staticmethod
+    def _parse_optional(cell) -> Optional[str]:
+        text = cell.text.strip()
+        if text and text != 'N/A':
+            return text
+        return None
+
+    @staticmethod
+    def submission_id(cell):
+        return int(cell.text.rstrip('#'))
+
+    @staticmethod
+    def submission_time(cell):
+        # NOTE timezone is not set
+        return datetime.strptime(cell.text, TIME_FORMAT)
+
+    @staticmethod
+    def submission_tests(cell):
+        value = _CellParsers._parse_optional(cell)
+        return int(value) if value is not None else None
+
+    @staticmethod
+    def submission_score(cell):
+        value = _CellParsers._parse_optional(cell)
+        if value is None:
+            return None
+        return int(re.sub(r'=.*', '', value))  # score may include penalty
+
+    @staticmethod
+    def submission_source(cell):
+        source_link = cell.find('a')['href']
+        return source_link.replace(
+            f'action={Page.VIEW_SOURCE.value}',
+            f'action={Page.DOWNLOAD_SOURCE.value}'
+        )
+
+    @staticmethod
+    def submission_report(cell):
+        report_link = cell.find('a')
+        return report_link['href'] if report_link is not None else None
+
+
+@dataclass(frozen=True)
 class Submission:
-    def __init__(self, row):
+    id: int = _parse_field(_CellParsers.submission_id)
+    # `_skip_field`s are unused, also parsing of these fields may be unstable
+    time: datetime = _skip_field(parser=_CellParsers.submission_time)
+    size: int = _skip_field()
+    problem: str
+    compiler: str
+    status: str
+    tests_passed: Optional[int] = _skip_field(parser=_CellParsers.submission_tests)
+    score: Optional[int] = _skip_field(parser=_CellParsers.submission_score)
+    source: str = _parse_field(_CellParsers.submission_source)
+    report: Optional[str] = _parse_field(_CellParsers.submission_report)
+
+    @classmethod
+    def parse(cls, row):
+
+        def parse_field(field, cell):
+            # this function is  never called on `_skip_field`s
+            parser = field.metadata.get('parser')
+            if not parser:
+                # NOTE will not work with Optional types
+                return field.type(cell.text)
+            return parser(cell)
+
         cells = row.find_all('td')
-        self.id = int(cells[0].text.rstrip('#'))
-        self.problem = cells[3].text
-        self.compiler = cells[4].text
-        self.status = cells[5].text
-        self.source = cells[8].find('a')['href'].replace(f'action=36', f'action=91')
-        report_link = cells[9].find('a')
-        self.report = report_link['href'] if report_link is not None else None
+        data = {
+            field.name: parse_field(field, cell)
+            for field, cell in zip(fields(cls), cells) if field.init
+        }
+        return cls(**data)
 
     def short_status(self):
         if self.status == Status.REVIEW:
@@ -173,7 +262,10 @@ class Report:
                 author_cell, comment, *_ = row.find_all('td')
                 author = next(line for line in author_cell.text.splitlines() if line)
                 # if the comment contains newlines
-                yield from f'Comment by {author}: {comment.text.strip()}\n'.splitlines(keepends=True)
+                yield from (
+                    f'Comment by {author}: {comment.text.strip()}\n'
+                    .splitlines(keepends=True)
+                )
 
         if comments:
             self.lines = list(comm_format(comments))
@@ -294,15 +386,20 @@ class Deadlines:
         return result
 
     @staticmethod
-    def parse(text):
-        """Parse MSK datetime string (obtained from a problem page) and convert it to UTC"""
-        dt = datetime.strptime(text, '%Y/%m/%d %H:%M:%S')
-        return dt.replace(tzinfo=SERVER_TZ).astimezone(MSK_TZ)
+    def parse(text, server_tz):
+        """Parse datetime string (obtained from a problem page) and convert it to UTC"""
+        dt = datetime.strptime(text, TIME_FORMAT)
+        return dt.replace(tzinfo=server_tz).astimezone(MSK_TZ)
 
 
 class ProblemInfo:
     """Subset of task info table used for max score estimation"""
-    def __init__(self, full_score: int, run_penalty: int, current_penalty: int, deadlines: Deadlines):
+    def __init__(  # TODO use dataclass?
+        self,
+        full_score: int,
+        run_penalty: int, current_penalty: int,
+        deadlines: Deadlines
+    ):
         self.full_score = full_score
         self.run_penalty = run_penalty
         self.current_penalty = current_penalty
@@ -319,7 +416,10 @@ class ProblemInfo:
         return self.deadlines.is_close(self.active_deadline())
 
     def past_deadline(self):
-        return self.deadlines.hard is not None and datetime.now(tz=timezone.utc) > self.deadlines.hard
+        return (
+            self.deadlines.hard is not None
+            and datetime.now(tz=timezone.utc) > self.deadlines.hard
+        )
 
 
 class ContestInfo:
@@ -339,6 +439,7 @@ class ContestInfo:
     def __getattr__(self, name):
         return getattr(self.first_problem, name)
 
+
 class FullProblem(SummaryProblem):
     keep_info = ['Time limit:', 'Real time limit:', 'Memory limit:']
 
@@ -392,10 +493,15 @@ class FullProblem(SummaryProblem):
         input_title = html.find('h4', text='Input')
         if input_title is not None:
             input_data = input_title.find_next('pre').text
+            # Add trailing newline, like in vim
+            if not input_data.endswith('\n'):
+                input_data += '\n'
 
         output_title = html.find('h4', text='Output')
         if output_title is not None:
             output_data = output_title.find_next('pre').text
+            if not output_data.endswith('\n'):
+                output_data += '\n'
 
         return input_data, output_data
 
@@ -493,6 +599,21 @@ class FullProblem(SummaryProblem):
         converter.pad_tables = True
         return converter.handle(str(self._html))
 
+    def attachments(self):
+        """Returns a dict of attachments in form of {filename: url}"""
+        if not self.statement_available():
+            return {}
+        attachments = {}
+        for tag in self._html.find_all(['a', 'img']):
+            url = tag['href'] if tag.name == 'a' else tag['src']
+            parts = urlsplit(url)
+            if parts.netloc != Links.HOST:
+                continue
+            query = parse_qs(parts.query)
+            if 'file' in query:
+                attachments[query['file'][0]] = url
+        return attachments
+
 
 def get_contest_id(group_id: str) -> int:
     return CONTEST_ID_BY_GROUP.get(group_id, None)
@@ -518,7 +639,7 @@ def get_contest_url_with_creds(auth_data):
 def ejudge_summary(session):
     from bs4 import BeautifulSoup
 
-    page = session.get(Links.SUMMARY)
+    page = session.get_page(Page.SUMMARY)
 
     soup = BeautifulSoup(page.content, 'html.parser')
 
@@ -543,7 +664,7 @@ def ejudge_summary(session):
 def ejudge_standings(session):
     from bs4 import BeautifulSoup
 
-    page = session.get(Links.USER_STANDINGS)
+    page = session.get_page(Page.USER_STANDINGS)
     soup = BeautifulSoup(page.content, 'html.parser')
 
     title = soup.find(class_='main_phrase').text
@@ -615,14 +736,14 @@ def to_task_score(contest, cell):
 def ejudge_submissions(session):
     from bs4 import BeautifulSoup
 
-    page = session.get(Links.SUBMISSIONS, params={'all_runs': 1})
+    page = session.get_page(Page.SUBMISSIONS, params={'all_runs': 1})
 
     soup = BeautifulSoup(page.content, 'html.parser')
 
     sub_table = soup.find('table', {'class': 'table'})
     if sub_table is None:
         return []
-    submissions = [Submission(row) for row in sub_table.find_all('tr')[1:]]
+    submissions = [Submission.parse(row) for row in sub_table.find_all('tr')[1:]]
     submissions.sort(key=lambda x: x.problem)
     return {
         problem: list(subs) for problem, subs in groupby(submissions, lambda x: x.problem)
@@ -643,6 +764,30 @@ def ejudge_report(link, session):
     return Report(comments, tests)
 
 
+def ejudge_timezone(session):
+    from bs4 import BeautifulSoup
+
+    page = session.get_page(Page.MAIN_PAGE)
+    soup = BeautifulSoup(page.content, 'html.parser')
+    table = soup.find('table', {'class': 'info-table-line'})
+    if table is None:
+        raise ParseError('Cannot parse server time')
+    offset_hours = None
+    for row in table.find_all('tr'):
+        cells = row.find_all('td')
+        if len(cells) < 2:
+            continue
+        key, value, *_ = cells
+        if 'Server time' in key.text:
+            server_time = datetime.strptime(value.text, TIME_FORMAT)
+            utc_time = datetime.utcnow()
+            offset_hours = round((server_time - utc_time).total_seconds() / 3600)
+            break
+    if offset_hours is None:
+        raise ParseError('Cannot parse server time')
+    return timezone(timedelta(hours=offset_hours))
+
+
 def get_contest_deadlines(session, summary, no_cache):
     from kks.util.storage import Cache
 
@@ -658,8 +803,11 @@ def get_contest_deadlines(session, summary, no_cache):
         if no_cache:
             for problem in summary:
                 cache.erase(CacheKeys.deadline(problem.contest()))
+            cache.erase(CacheKeys.server_tz)
 
-        problems = update_cached_problems(cache, names, session, problems=first_problems, summary=summary)
+        problems = update_cached_problems(
+            cache, names, session, problems=first_problems, summary=summary
+        )
 
     return [ContestInfo(name, problem) for name, problem in zip(contest_names, problems)]
 
@@ -671,12 +819,12 @@ def update_cached_problems(cache, names, session, problems=None, summary=None):
     def cached_problem(problem):
         return BaseProblem(problem.short_name, problem.href)
 
-    problem_list = cache.get('problem_links', [])  # we can avoid loading summary
+    problem_list = cache.get(CacheKeys.problem_links, [])  # we can avoid loading summary
     if len(problem_list) != len(names) or \
             any(problem.short_name != name for problem, name in zip(problem_list, names)):
         problem_list = summary or ejudge_summary(session)
         problem_list = [cached_problem(p) for p in problem_list]
-        cache.set('problem_links', problem_list)
+        cache.set(CacheKeys.problem_links, problem_list)
 
     if problems is not None:
         problem_list = [problem for problem in problem_list if problem.short_name in problems]
@@ -687,7 +835,9 @@ def update_cached_problems(cache, names, session, problems=None, summary=None):
             pbar.update(1)
             return result
 
-        return [with_progress(get_problem_info, problem, cache, session) for problem in problem_list]
+        return [
+            with_progress(get_problem_info, problem, cache, session) for problem in problem_list
+        ]
 
 
 def get_problem_info(problem, cache, session):
@@ -695,8 +845,8 @@ def get_problem_info(problem, cache, session):
 
     # NOTE penalties are assumed to be the same for all tasks in a contest, it may be wrong
 
-    full_scores = cache.get('full', {})
-    run_penalties = cache.get('run', {})
+    full_scores = cache.get(CacheKeys.full_scores, {})
+    run_penalties = cache.get(CacheKeys.run_penalties, {})
 
     full_score = full_scores.get(problem.short_name)
     run_penalty = run_penalties.get(problem.short_name)
@@ -707,7 +857,10 @@ def get_problem_info(problem, cache, session):
     deadline_key = CacheKeys.deadline(problem.contest())
     deadlines = cache.get(deadline_key)
 
-    need_loading = any(field is None for field in [full_score, run_penalty, current_penalty, deadlines])  # new problem or need to update penalty
+    # new problem or need to update penalty
+    need_loading = any(
+        field is None for field in [full_score, run_penalty, current_penalty, deadlines]
+    )
 
     if not need_loading:
         return ProblemInfo(full_score, run_penalty, current_penalty, deadlines)
@@ -716,10 +869,11 @@ def get_problem_info(problem, cache, session):
         result = ProblemInfo(full_score, run_penalty, current_penalty, deadlines)
         full_scores[problem.short_name] = full_score
         run_penalties[problem.short_name] = run_penalty
-        cache.set('full', full_scores)
-        cache.set('run', run_penalties)
+        cache.set(CacheKeys.full_scores, full_scores)
+        cache.set(CacheKeys.run_penalties, run_penalties)
 
-        if result.past_deadline() or problem.contest().startswith('kr'):  # kr tasks don't have soft deadlines, so nothing should expire
+        # kr tasks don't have soft deadlines, so their data should not expire
+        if result.past_deadline() or problem.contest().startswith('kr'):
             expiration = None
         else:
             if deadlines.soft is not None:
@@ -760,9 +914,9 @@ def get_problem_info(problem, cache, session):
             elif key.text == 'Current penalty:':
                 current_penalty = int(value.text)
             elif key.text == 'Next soft deadline:':
-                deadlines.soft = Deadlines.parse(value.text)
+                deadlines.soft = Deadlines.parse(value.text, get_server_tz(cache, session))
             elif key.text == 'Deadline:':
-                deadlines.hard = Deadlines.parse(value.text)
+                deadlines.hard = Deadlines.parse(value.text, get_server_tz(cache, session))
 
     if not full_score_found:
         try:
@@ -770,13 +924,22 @@ def get_problem_info(problem, cache, session):
         except APIError as e:
             click.secho(f'Cannot get problem info ({problem.short_name}): {e}', err=True)
             problem_status = {}
-        full_score = problem_status.get('full_score', 0)  # NOTE is equal to 1 for running / testing kr contests
+        # NOTE for running / testing kr contests full_score == 1
+        full_score = problem_status.get('full_score', 0)
         run_penalty = problem_status.get('run_penalty', 0)
         # API never tells the deadlines and current penalty
 
     return update_cache(full_score, run_penalty, current_penalty, deadlines)
 
 
+def get_server_tz(cache, session):
+    tz = cache.get(CacheKeys.server_tz, None)
+    if tz is None:
+        tz = ejudge_timezone(session)
+        cache.set(CacheKeys.server_tz, tz, expiration=timedelta(days=7))
+    return tz
+
+
 def chunks(iterable, chunk_size):
     for i in range(0, len(iterable), chunk_size):
         yield iterable[i:i + chunk_size]
