@@ -40,11 +40,13 @@ class SubmissionResult:
     def parse_status(cls, run_status):
         if run_status.status in [RunStatus.OK, RunStatus.PENDING_REVIEW]:
             return cls.ok(str(run_status))
-        if run_status.status in [RunStatus.ACCEPTED, RunStatus.PENDING]:  # PENDING == "Pending check" =?= ACCEPTED
+        # PENDING == "Pending check" =?= ACCEPTED (??)
+        if run_status.status in [RunStatus.ACCEPTED, RunStatus.PENDING]:
             return cls.check(str(run_status))
         if run_status.status in [RunStatus.CE, RunStatus.STYLE_ERR]:
             return cls.fail(run_status.with_compiler_output())
-        return cls.fail(run_status.with_tests(failed_only=True))  # there can be 100+ passed tests and a few failed
+        # Hide OK's. There can be 100+ passed tests and a few failed.
+        return cls.fail(run_status.with_tests(failed_only=True))
 
 
 def get_lang(available, all_langs):
@@ -89,7 +91,10 @@ def submit_solution(session, file, prob_name, timeout):
     if not problem_status.get('is_submittable'):
         return SubmissionResult.fail('Cannot submit a solution for this problem')
     ok_fields = ['is_solved', 'is_pending', 'is_pending_review', 'is_accepted']
-    if any(field in problem_status for field in ok_fields) and not click.confirm('This problem was already solved! Submit anyway?'):
+    if (
+        any(field in problem_status for field in ok_fields)
+        and not click.confirm('This problem was already solved! Submit anyway?')
+    ):
         return SubmissionResult.fail('Cancelled by user')
 
     lang = get_lang(problem.get('compilers', []), contest['compilers'])
